import { AnyArray, IsTuple, IsUnknown } from "ts-essentials";
export type DeepImmutable<T> = T extends Date ? ReadonlyDate : T extends URL ? ReadonlyURL : T extends URLSearchParams ? ReadonlyURLSearchParams : T extends Map<infer K, infer V> ? ImmutableMap<DeepImmutable<K>, DeepImmutable<V>> : T extends ReadonlyMap<infer K, infer V> ? ImmutableMap<DeepImmutable<K>, DeepImmutable<V>> : T extends ImmutableMap<infer K, infer V> ? ImmutableMap<DeepImmutable<K>, DeepImmutable<V>> : T extends WeakMap<infer K, infer V> ? ReadonlyWeakMap<DeepImmutable<K>, DeepImmutable<V>> : T extends ReadonlyWeakMap<infer K, infer V> ? ReadonlyWeakMap<DeepImmutable<K>, DeepImmutable<V>> : T extends Set<infer U> ? ImmutableSet<DeepImmutable<U>> : T extends ReadonlySet<infer U> ? ImmutableSet<DeepImmutable<U>> : T extends ImmutableSet<infer U> ? ImmutableSet<DeepImmutable<U>> : T extends WeakSet<infer U> ? ReadonlyWeakSet<DeepImmutable<U>> : T extends ReadonlyWeakSet<infer U> ? ReadonlyWeakSet<DeepImmutable<U>> : T extends Promise<infer U> ? ReadonlyPromise<DeepImmutable<U>> : T extends AnyArray<infer U> ? T extends IsTuple<T> ? {
    readonly [K in keyof T]: DeepImmutable<T[K]>;
} : ImmutableArray<DeepImmutable<U>> : T extends {} ? {
    readonly [K in keyof T]: DeepImmutable<T[K]>;
} : IsUnknown<T> extends true ? unknown : Readonly<T>;
export type ReadonlyPartial<T> = Readonly<Partial<T>>;
export type ReadonlyRequired<T> = Readonly<Required<T>>;
export type ReadonlyPick<T, K extends keyof T> = Readonly<Pick<T, K>>;
export type ReadonlyRecord<K extends string | number | symbol, T> = Readonly<Record<K, T>>;
export type ImmutableShallow<T extends {}> = {
    readonly [P in keyof T & {}]: T[P];
};
export type ImmutableArray<T> = ImmutableShallow<ReadonlyArray<T>>;
export type ReadonlyPromise<T> = Readonly<Promise<T>>;
export type ImmutableSet<T> = Readonly<ReadonlySet<T>>;
export type ImmutableMap<K, V> = Readonly<ReadonlyMap<K, V>>;
/**
 * Drop keys `K` from `T`, where `K` must exist in `T`.
 *
 * @see https://github.com/pelotom/type-zoo
 * @see https://github.com/Microsoft/TypeScript/issues/12215#issuecomment-377567046
 */
export type OmitStrict<T, K extends keyof T> = T extends any ? Pick<T, Exclude<keyof T, K>> : never;
export type ReadonlyURLSearchParams = Readonly<OmitStrict<URLSearchParams, "append" | "delete" | "set" | "sort">> & {
    readonly [Symbol.iterator]: () => IterableIterator<readonly [string, string]>;
};
export declare const readonlyURLSearchParams: (init?: readonly (readonly string[])[] | string[][] | Record<string, string> | string | URLSearchParams | ReadonlyURLSearchParams) => ReadonlyURLSearchParams;
export type ReadonlyURL = Readonly<OmitStrict<URL, "searchParams">> & {
    readonly searchParams: ReadonlyURLSearchParams;
};
export declare const readonlyURL: (url: string, base?: string | URL | ReadonlyURL) => ReadonlyURL | undefined;
export type ReadonlyDate = Readonly<OmitStrict<Date, "setTime" | "setMilliseconds" | "setUTCMilliseconds" | "setSeconds" | "setUTCSeconds" | "setMinutes" | "setUTCMinutes" | "setHours" | "setUTCHours" | "setDate" | "setUTCDate" | "setMonth" | "setUTCMonth" | "setFullYear" | "setUTCFullYear">>;
export declare const readonlyDate: (value: number | string | Date | ReadonlyDate) => ReadonlyDate;
export declare const validReadonlyDate: (value: number | string | Date | ReadonlyDate) => ReadonlyDate | undefined;
export declare const readonlyNow: () => number;
export type ReadonlyWeakSet<T extends object> = Readonly<OmitStrict<WeakSet<T>, "add" | "delete">>;
export type ReadonlyWeakMap<K extends object, V> = Readonly<OmitStrict<WeakMap<K, V>, "delete" | "set">>;
export declare const readonlySet: <T>(values: Iterable<T>) => Readonly<ReadonlySet<T>>;
export declare const readonlyWeakSet: <T extends object>(values: Iterable<T>) => Readonly<Pick<WeakSet<T>, typeof Symbol.toStringTag | "has">>;
export declare const readonlyMap: <K, V>(values: Iterable<readonly [K, V]>) => Readonly<ReadonlyMap<K, V>>;
export declare const readonlyWeakMap: <K extends object, V>(values: Iterable<readonly [K, V]>) => Readonly<Pick<WeakMap<K, V>, typeof Symbol.toStringTag | "has" | "get">>;
